package com.qmth.themis.business.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.ExamRecordCacheBean;
import com.qmth.themis.business.dao.TOeExamRecordMapper;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.RedisUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 考试记录 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TOeExamRecordServiceImpl extends ServiceImpl<TOeExamRecordMapper, TOeExamRecord> implements TOeExamRecordService {

    @Resource
    TOeExamRecordMapper tOeExamRecordMapper;

    @Resource
    RedisUtil redisUtil;

    /**
     * 获取考试未完列表
     *
     * @param studentId
     * @param examId
     * @param orgId
     * @return
     */
    @Override
    public Map getUnFinishExam(Long studentId, Long examId, Long orgId) {
        Map unFinishExam = tOeExamRecordMapper.getUnFinishExam(studentId, examId, orgId);
        if (Objects.nonNull(unFinishExam)) {
            Map finalMap = new HashMap();
            Map<String, Object> waitingMap = new HashMap();
            waitingMap.put("id", unFinishExam.get("id"));
            waitingMap.put("examActivityId", unFinishExam.get("examActivityId"));
            waitingMap.put("name", unFinishExam.get("name"));
            waitingMap.put("mode", unFinishExam.get("mode"));
            waitingMap.put("preNotice", unFinishExam.get("preNotice"));
            waitingMap.put("preNoticeStaySeconds", unFinishExam.get("preNoticeStaySeconds"));
            waitingMap.put("postNotice", unFinishExam.get("postNotice"));
            finalMap.put("waiting", waitingMap);

            Map<String, Object> activityMap = new HashMap();
            activityMap.put("recordId", unFinishExam.get("recordId"));
            activityMap.put("minDurationSeconds", unFinishExam.get("minDurationSeconds"));
            activityMap.put("clientVideoPush", unFinishExam.get("clientVideoPush"));
            activityMap.put("cameraPhotoUpload", unFinishExam.get("cameraPhotoUpload"));
            activityMap.put("courseCode", unFinishExam.get("courseCode"));
            activityMap.put("wxappPhotoUpload", unFinishExam.get("wxappPhotoUpload"));
            activityMap.put("inProcessLivenessRetryCount", unFinishExam.get("inProcessLivenessRetryCount"));
            activityMap.put("examStudentId", unFinishExam.get("examStudentId"));
            activityMap.put("inProcessLivenessVerify", unFinishExam.get("inProcessLivenessVerify"));
            activityMap.put("inProcessFaceStrangerIgnore", unFinishExam.get("inProcessFaceStrangerIgnore"));
            activityMap.put("wxappVideoRecord", unFinishExam.get("wxappVideoRecord"));
            activityMap.put("inProcessFaceVerify", unFinishExam.get("inProcessFaceVerify"));
            activityMap.put("courseName", unFinishExam.get("courseName"));
            activityMap.put("wxappVideoPush", unFinishExam.get("wxappVideoPush"));
            activityMap.put("inProcessLivenessIntervalSeconds", unFinishExam.get("inProcessLivenessIntervalSeconds"));
            activityMap.put("prepareSeconds", unFinishExam.get("prepareSeconds"));
            activityMap.put("fouceFinish", unFinishExam.get("fouceFinish"));
            activityMap.put("entryAuthenticationPolicy", unFinishExam.get("entryAuthenticationPolicy"));
            activityMap.put("maxDurationSeconds", unFinishExam.get("maxDurationSeconds"));
            activityMap.put("examCount", unFinishExam.get("examCount"));
            activityMap.put("inProcessLivenessJudgePolicy", unFinishExam.get("inProcessLivenessJudgePolicy"));
            activityMap.put("id", unFinishExam.get("examActivityId"));

            finalMap.put("activity", activityMap);
            return finalMap;
        } else {
            return unFinishExam;
        }
    }

    @Transactional
    @Override
    public Long saveByPrepare(Long examId, Long examActivityId, Long examStudentId, Long paperId, Integer serialNumber) {
        TOeExamRecord er = new TOeExamRecord();
        er.setExamId(examId);
        er.setExamActivityId(examActivityId);
        er.setExamStudentId(examStudentId);
        er.setPaperId(paperId);
        er.setSerialNumber(serialNumber);
        er.setFirstPrepareTime(new Date());
        er.setStatus(0);
        saveOrUpdate(er);
        ExamRecordCacheBean erCache = new ExamRecordCacheBean();
        erCache.setId(er.getId());
        erCache.setExamId(examId);
        erCache.setExamActivityId(examActivityId);
        erCache.setExamStudentId(examStudentId);
        erCache.setPaperId(paperId);
        erCache.setSerialNumber(serialNumber);
        erCache.setFirstPrepareTime(new Date());
        erCache.setStatus(0);
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(er.getId()), erCache);
        return er.getId();
    }
}
